
% extractsection.m
%
% Extract a section from a *.mat file, which has
% been created from a *.ice file by "ice2mat.m".
% These files can be a grab-bag of ctd and bottle
% data for many sections.

% first, plot all the data
filename=input('Filename? [*.mat] ','s');
eval(['load ',filename]);
clf;
plot(lon,lat,'.k','markersize',6);
overlay;
axis([min(lon)-2 max(lon)+2 min(lat)-2 max(lat)+2]);
disp('Use mouse to zoom in to desired section.');
disp('  (press ENTER in this window when done)');
zoom on;
pause;

% now allow the user to define a polygon to
% geographically isolate the casts
disp('Use mouse to define a polygon which surrounds');
disp(' the desired casts (ENTER when done).');
[lonsect,latsect]=ginput;
q=find(isinpoly(lon,lat,lonsect,latsect)==1);
subplot(211);
plot(lon(q),lat(q),'.k','markersize',10);
set(gca,'dataaspectratio',[1 1 1]);
xlabel('^\circE');ylabel('^\circN');
subplot(212);
timesect=(time-julian([1900,1,1,0,0,0]))/365.24+1900;
plot(lon(q),timesect(q),'*k');
xlabel('^\circE');ylabel('Time (year)');
fprintf('Max time - min time: %d days.\n', ...
  round(max(time(q))-min(time(q))) );

% now allow user to isolate a particular time
% (in case multiple sections occupied the same area).
isotime=input('Keep all sections shown here? [y/n]','s');
if (isotime=='n');
  disp('Zoom on; type ENTER in this window to proceed.');
  zoom on;
  pause;
  disp('Mouse-click twice on time plot to define times');
  disp('  (vertical coords of clicks) to retain.');
  [isox isoy]=ginput(2);
  q2=find(timesect(q)>=min(isoy) & timesect(q)<=max(isoy));
  q=q(q2);
  subplot(211);
  plot(lon(q),lat(q),'.k','markersize',10);
  set(gca,'dataaspectratio',[1 1 1]);
  xlabel('^\circE');ylabel('^\circN');
  subplot(212);
  plot(lon(q),timesect(q),'*k');
  xlabel('^\circE');ylabel('Time (year)');
  fprintf('Max time - min time: %d days.\n', ...
    round(max(time(q))-min(time(q))) );
end;

% if all is well at this point, we have a vector
% q which contains the indices for the positions
% and times (i.e. the navigation lines) of the
% desired casts.  The hydrographic data for
% nav. line q(i) runs from q(i)+1 to the next
% nav. line (which will have non-NaN lon,lat,time).
%
% Now identify the indices for all hydrographic data
% corresponding to the desired casts, and discard
% the rest.

q2=find(~isnan(lon+lat+time));
ind=[];
for i=1:length(q);
  nextcastind=find(q2>q(i));
  if (length(nextcastind))
    nextcastind=q2(nextcastind(1));
    ind=[ind;(q(i):nextcastind-1)'];
  else
    ind=[ind;(q(i):length(time))'];
  end;
end;

% truncate data to retain only the desired casts

data=whos;	% create a struct array with data names, size
lonsize=length(lon);
for i=1:size(data,1);
  varname=getfield(data(i,:),'name');
  varsize=max(getfield(data(i,:),'size'));
  if (varsize==lonsize);
    eval([varname,'=',varname,'(ind,:);']);
  end;
end;

% clean up
clear q q2 varname varsize ind i isox isoy;
clear lonsect latsect isotime timesect;

% now create separate ctd and bottle files.
%filename2=input('Prefix for this section? ','s');
%if (filename2==filename)
  %fprintf('Warning!  This name will overwrite %s.mat.\n',filename);
  %disp('  Adding suffix "new".');
  %filename2=[filename2,'new'];
%end;

q=find(isfinite(time+lon+lat));
q=[q;length(time)+1];

new_pres=(0:2:max(ctdpres))';
data=whos;      % create a struct array with data names, size
for i=1:size(data,1);
  varname=getfield(data(i,:),'name');
  if (length(varname)>3');
    if (varname(1:3)=='ctd' | varname(1:3)=='bot');
      eval(['new_',varname,'=NaN*ones', ...
	'(length(new_pres),length(q)-1);']);
      for j=1:length(q)-1;
	eval(['var=',varname,'(q(j)+1:q(j+1)-1);']);
        if (varname(1:3)=='ctd');
          p=ctdpres(q(j)+1:q(j+1)-1);
        else
          p=botpres(q(j)+1:q(j+1)-1);
        end;
        qq=find(isfinite(var));
        if (length(qq)>3)
          eval(['new_',varname,'(:,j)=interp1(', ...
	  'p(qq)+cumsum(p(qq))/1e5,var(qq),', ...
	  'new_pres,''linear'');']);
        end;
      end;
    end;
  end;
end;
new_botpres=new_pres;new_ctdpres=new_pres;
new_lon=lon(q(1:length(q)-1));
new_lat=lat(q(1:length(q)-1));

 now create separate ctd and bottle files.
filename2=input('Prefix for this section? ','s');
if (filename2==filename)
  fprintf('Warning!  This name will overwrite %s.mat.\n',filename);
  disp('  Adding suffix "new".');
  filename2=[filename2,'new'];
end;
eval(['save ',filename2]);



