%filename='nansen.ice';

% ice2mat
%
% convert a file in ICES format
% (see http://www.ices.dk/ocean)
% to Matlab format.
%
% NOTE: this file assumes that one has removed
% the control-M characters from the *.ice file.
% This can be done via "dos2unix filename.ice".

if (filename(length(filename)-3: ...
	length(filename))~='.ice')
  filename=[filename,'.ice'];
end;

S=[];
counter=[1 5000];
flength=1;
while (flength)
  eval(['!sed -n ''',num2str(counter(1)),',', ...
	num2str(counter(2)),'p'' ',filename,' >! temp']);

  eval(['!ls -s temp >! filelength']);
  filelength=textread('filelength','%c','whitespace','\n')';
  filelen2=filelength;
  for i=1:length(filelength)-4+1;
    if (strcmp('temp',filelength(i:i-1+4)));
      filelen2(i:i-1+4)=[];
    end;
  end;
  flength=str2num(filelen2)

  lst=['s=textread(''temp'','];
  lst=[lst,'''%c'',''whitespace'',''\n'');'];
  fprintf('Loading data . . . ');
  eval(lst);
  fprintf('done.\n');
  clear lst;
  s=reshape(s,[80 length(s)/80])';
  S=[S;s]; clear s;

  counter=counter+5000;
end;

clear counter filelen2 filelength flength i;

ctdpres=NaN*ones(size(S,1),1);
ctdtemp=ctdpres;ctdsal=ctdpres;ctdoxy=ctdpres;
stnnbr=ctdpres;
lon=ctdpres;lat=ctdpres;year=ctdpres;month=ctdpres;
day=ctdpres;hour=ctdpres;minute=ctdpres;bottom=ctdpres;
country=NaN*ones(size(S,1),2);
ship=country;
% hydrochemistry variables
botpres=ctdpres; bottemp=ctdpres;
botsal=ctdpres; botoxy=ctdpres;
botphos=ctdpres; bottotphos=ctdpres;
botsil=ctdpres; 
botnitrate=ctdpres; botnitrite=ctdpres; 
botnitraite=ctdpres;	 % nitrate+nitrite
botamm=ctdpres;
bottotN=ctdpres; botHS=ctdpres;
botpH=ctdpres; botalk=ctdpres;
botchlor=ctdpres;

for i=1:size(S,1);
  
  if ~mod(i,100);
    fprintf('%d of %d.\n',i,size(S,1));
  end;

  s=S(i,:);
  if (s(79:80)=='0J');	% hydromaster record
    country(i,:)=s(1:2);
    ship(i,:)=s(3:4);
    stnnbr(i)=icechar2num(s(5:8),0);
    quad=str2num(s(18));
    if (quad==0)
      latmult=1; lonmult=1;
    elseif (quad==1)
      latmult=1; lonmult=-1;
    elseif (quad==2)
      latmult=-1; lonmult=1;
    else
      latmult=-1; lonmult=-1;
    end;

    tdeg=icechar2num(s(9:10),1);
    tmin=icechar2num(s(11:12),1);
    tm=icechar2num(s(65:66),0);
    tvar=latmult*(tdeg+(tmin+tm/100)/60);
    lat(i)=tvar;

    tdeg=icechar2num(s(13:15),1);
    tmin=icechar2num(s(16:17),1);
    tm=icechar2num(s(67:68),0);
    tvar=lonmult*(tdeg+(tmin+tm/100)/60);
    lon(i)=tvar;

    yr=icechar2num(s(19:21),1);
    if ~length(yr) yr=NaN; end;
    if (yr>900) yr=yr+1000;
    else yr=yr+2000; end;
    year(i)=yr;
    month(i)=icechar2num(s(22:23),1);
    day(i)=icechar2num(s(24:25),1);
    hour(i)=icechar2num(s(26:27),0);
    minute(i)=icechar2num(s(69:70),0);
    bottom(i)=icechar2num(s(28:31),1);

    ctdpres(i)=-999;ctdtemp(i)=-999;
    ctdsal(i)=-999;ctdoxy(i)=-999;

  elseif (s(80)=='3');	% hydrography

    z=icechar2num(s(28:31),1);
    z2=icechar2num(s(42:43),0);
    z=z+z2/100; clear z2;
    depthorpress=s(41);
    if (depthorpress==' ')
      depthorpress='d';
    end;
    if (depthorpress~='p' & isfinite(z));
      z=sw_pres(z,last(lat(isfinite(lat))));
    end;
    ctdpres(i)=z;

    if (s(32)=='}')
      z=-icechar2num(s(33:35),1);
    else
      z=icechar2num(s(32:35),1);
    end;
    z2=icechar2num(s(45:46),0);
    z=z+z2/100; clear z2;
    ctdtemp(i)=z/100;
    
    z=icechar2num(s(36:40),1);
    z2=icechar2num(s(48:49),0);
    z=z+z2/100; clear z2;
    ctdsal(i)=z/1e3;

   % oxygen.  If s(79)=='K', units are cm^3/kg
   % 		(two implied decimals).
   %	Otherwise, units are cm^3/dm^3.
   % 	Use ox_units.m to convert to micromol/kg.
    z=icechar2num(s(58:60),1);
    sigfigs=length(num2str(z));
    if (s(79)~='K')
	z=ox_units(z,ctdsal(i), ...
  	  sw_ptmp(ctdsal(i),ctdtemp(i),ctdpres(i),0));
    else
        z=z/1e3*sw_dens(ctdsal(i),ctdtemp(i),ctdpres(i));
        z=ox_units(z,ctdsal(i), ...
          sw_ptmp(ctdsal(i),ctdtemp(i),ctdpres(i),0));
    end;
    ctdoxy(i)=keepsigfigs(z/1e2,sigfigs);

  elseif (s(80)=='6');  % hydrochemistry

    % the number of implied decimal points in the bottle data 
    % depends on whether position 79-80 of the record is "76"
    % or "P6"/"56".  Set a flag "linetype" to be 1 for "76",
    % 0 for "P6"/"56".
    %
    % In comments below, the statement
    %     4 (*3) implied decimals
    % means 4 implied for linetype==1, 3 for linetype==0.


    if (s(79:80)=='76') linetype=0;
    elseif (s(79:80)=='P6' | s(79:80)=='56') linetype=1'
    else
      s
      disp('WARNING: line type undefined in ice2mat.m');
    end;

    z=icechar2num(s(28:31),1);
    if (depthorpress~='p' & isfinite(z));
      z=sw_pres(z,last(lat(isfinite(lat))));
    end;
    botpres(i)=z;

    if (s(32)=='}')
      z=-icechar2num(s(33:35),1);
    else
      z=icechar2num(s(32:35),1);
    end;
    bottemp(i)=z/100;
    z=icechar2num(s(36:39),1);
    botsal(i)=z/100;

    % bottle values are in [units]/l unless s(78)=='K',
    % in which case they are [units]/kg.  Create 
    % "multfact", which will convert the s(78)~='K'
    % values to [units]/kg.  Note: round after converting,
    % to avoid misleading number of apparent signif. figures.
    multfact=1e3/sw_dens(botsal(i),bottemp(i),botpres(i));

    % oxygen.  If s(78)=='K', units are ml/kg 
    %	(two implied decimals).
    %    Otherwise, units are ml/l.
    %    Use ox_units.m to convert to micromol/kg.
    z=icechar2num(s(40:42),1);
    sigfigs=length(num2str(z));
    if (s(78)~='K')
        z=ox_units(z,botsal(i), ...
          sw_ptmp(botsal(i),bottemp(i),botpres(i),0));
    else % convert to ml/l, then use ox_units(ml/l -> mmol/kg)
        z=ox_units(z/multfact,botsal(i), ...
          sw_ptmp(botsal(i),bottemp(i),botpres(i),0));
    end;
    botoxy(i)=keepsigfigs(z/1e2,sigfigs);

    % phosphate.  micromol/l (mmol/kg if s(78)=='K').
    % 2(*1) implied decimals.
    z=icechar2num(s(43:45),1);
    sigfigs=length(num2str(z));
    if (s(78)~='K') z=z*multfact; end;
    botphos(i)=keepsigfigs(z/10^(1+linetype),sigfigs);
    % total phosphorus content, mmol/l (mmol/kg). 
    % 2(*1) implied decimals.
    z=icechar2num(s(46:48),1);
    sigfigs=length(num2str(z));
    if (s(78)~='K') z=z*multfact; end;
    bottotphos(i)=keepsigfigs(z/10^(1+linetype),sigfigs);

    % silicate, mmol/l (mmol/kg).
    % 1(*0) implied decimal.
    z=icechar2num(s(49:51),1);
    sigfigs=length(num2str(z));
    if (s(78)~='K') z=z*multfact; end;
    if (sigfigs>0)
      botsil(i)=keepsigfigs(z/10^linetype,sigfigs);
    end;

    % nitrate, mmol/l (mmol/kg). 1(*0) implied.
    z=icechar2num(s(52:54),1);
    sigfigs=length(num2str(z));
    if (s(78)~='K') z=z*multfact; end;
    botnitrate(i)=keepsigfigs(z/10^linetype,sigfigs);
    % nitrite, mmol/l (mmol/kg).  2(*1) implied.
    z=icechar2num(s(55:57),1);
    sigfigs=length(num2str(z));
    if (s(78)~='K') z=z*multfact; end;
    botnitrite(i)=keepsigfigs(z/10^(1+linetype),sigfigs);
    % nitrate+nitrite.  If nitrite is not reported,
    % but nitrate is, then the value in the nitrate field
    % is nitrate + nitrite.
    if ( isfinite(botnitrate(i)) & isnan(botnitrite(i)) )
      botnitraite(i)=botnitrate(i);
      botnitrate(i)=NaN;
    else
      botnitraite(i)=botnitrate(i)+botnitrite(i);
    end;
    % ammonium, mmol/l (mmol/kg). 1(*0) implied.
    z=icechar2num(s(58:60),1);
    sigfigs=length(num2str(z));
    if (s(78)~='K') z=z*multfact; end;
    botamm(i)=keepsigfigs(z/10^linetype,sigfigs);
    % total nitrogen, mmol/l (mmol/kg). 1(*0) implied.
    z=icechar2num(s(61:63),1);
    sigfigs=length(num2str(z));
    if (s(78)~='K') z=z*multfact; end;
    bottotN(i)=keepsigfigs(z/10^linetype,sigfigs);

    % Hydrogen Sulphide, mmol/l (mmol/kg). 1 implied.
    z=icechar2num(s(64:66),1);
    sigfigs=length(num2str(z));
    if (s(78)~='K') z=z*multfact; end;
    botHS(i)=keepsigfigs(z/10,sigfigs);

    % Hydrogen ion concentration.  2 implied.
    z=icechar2num(s(67:69),1);
    sigfigs=length(num2str(z));
    botpH(i)=keepsigfigs(z/100,sigfigs);

    % Alkalinity, milliequivalents/l (meq/kg) at 20 deg. C.
    % 3 decimal places implied.
    z=icechar2num(s(70:73),1);
    sigfigs=length(num2str(z));
    if (s(78)~='K') z=z*multfact; end;
    botalk(i)=keepsigfigs(z/1000,sigfigs);

    % Chlorophyll a, micrograms/l (micrograms/kg).
    % 1 implied; if 56 record, 2 implied.
    z=icechar2num(s(74:76),1);
    sigfigs=length(num2str(z));
    if (s(78)~='K') z=z*multfact; end;
    if (sum(s(79:80)=='56')==2) z=z/10; end;
    botchlor(i)=keepsigfigs(z/10,sigfigs);

  end;	% type of line (master,hydrography,hydrochemistry)

end;

% convert year, month, day, hour into decimal Julian days
time=julian([year month day hour minute zeros(size(minute))]);
clear year month day hour minute;

% loop through entries, filling the NaN'ed navigation
% entries by the last finite value.
%for i=2:length(lon);
  %if (isnan(lon(i)+lat(i)+stnnbr(i)+time(i)) );
    %stnnbr(i)=stnnbr(i-1);
    %lon(i)=lon(i-1);
    %lat(i)=lat(i-1);
    %time(i)=time(i-1);
    %country(i)=country(i-1);
    %ship(i)=ship(i-1);
    %bottom(i)=bottom(i-1);
  %end;
%end;

clear tvar S ans depthorpress i latmult lonmult;
clear quad s tm yr z q tdeg tmin ans multfact sigfigs;

filename=filename(1:length(filename)-4);

disp(['Saving ',filename,'.mat']);
eval(['save ',filename]);

clf;
plot(lon,lat,'.');
overlay;
axis([min(lon)-10 max(lon)+10 min(lat)-10 max(lat)+10]);
